Drupal.tagmap = new function() {
    var gmarkers = new Array();

    this.getClustererStyle = function(id, dir) {
        var styles = [
            [
                {
                    url: dir + 'people35.png',
                    height: 35,
                    width: 35,
                    opt_anchor: [16, 0],
                    opt_textColor: '#ff00ff',
                    opt_textSize: 10
                },
                {
                    url: dir + 'people45.png',
                    height: 45,
                    width: 45,
                    opt_anchor: [24, 0],
                    opt_textColor: '#ff0000',
                    opt_textSize: 11
                },
                {
                    url: dir + 'people55.png',
                    height: 55,
                    width: 55,
                    opt_anchor: [32, 0],
                    opt_textSize: 12
                }
            ],
            [
                {
                    url: dir + 'conv30.png',
                    height: 27,
                    width: 30,
                    anchor: [3, 0],
                    textColor: '#ff00ff',
                    opt_textSize: 10
                },
                {
                    url: dir + 'conv40.png',
                    height: 36,
                    width: 40,
                    opt_anchor: [6, 0],
                    opt_textColor: '#ff0000',
                    opt_textSize: 11
                },
                {
                    url: dir + 'conv50.png',
                    width: 50,
                    height: 45,
                    opt_anchor: [8, 0],
                    opt_textSize: 12
                }
            ],
            [
                {
                    url: dir + 'heart30.png',
                    height: 26,
                    width: 30,
                    opt_anchor: [4, 0],
                    opt_textColor: '#ff00ff',
                    opt_textSize: 10
                },
                {
                    url: dir + 'heart40.png',
                    height: 35,
                    width: 40,
                    opt_anchor: [8, 0],
                    opt_textColor: '#ff0000',
                    opt_textSize: 11
                },
                {
                    url: dir + 'heart50.png',
                    width: 50,
                    height: 44,
                    opt_anchor: [12, 0],
                    opt_textSize: 12
                }
            ]
        ];
        return styles[id];
    }

    /**
     * Set map options
     * @param vars
     * @param mapid
     */
    this.setMapOptions = function (vars, mapid) {
        var mapOptions = {};
        mapOptions.mapTypeControlOptions = {
            mapTypeIds : Drupal.tagmap.mapLayers(vars.baselayers, vars.layer),
            style : Drupal.tagmap.getMapLayersControlStyle(vars)
        };
        mapOptions.mapTypeControl = true;
        mapOptions.mapTypeId = Drupal.tagmap.defaultMapTypeId(mapid);
        mapOptions.scrollwheel = !vars.behavior.nomousezoom;
        mapOptions.center = Drupal.tagmap.setCenter(vars.initial_state.center);
        mapOptions.zoomControlOptions = {style : Drupal.tagmap.getZoomControlStyle(vars)};
        return mapOptions;
    };


    /**
     * Setting the map center. Check if user provided the default center
     * coordinates,
     * otherwise map center will be calculated on the basis of the
     * @param vars
     */
    this.setCenter = function(vars) {
        var center = new google.maps.LatLng(0, 0);
        if (vars) {
            center = vars(',');
            center = new google.maps.LatLng(center[0], center[1]);
        }
        return center;
    };

    /**
     * initialize map layers in map control
     * @param selectedLayers
     * @param bonusLayers
     */
    this.mapLayers = function(selectedLayers, bonusLayers) {
        var mapTypes = {
            'map' : google.maps.MapTypeId.ROADMAP,
            'satellite' : google.maps.MapTypeId.SATELLITE,
            'hybrid' : google.maps.MapTypeId.HYBRID,
            'physical' : google.maps.MapTypeId.TERRAIN
        };
        var layers = [];
        for (layer in selectedLayers) {
            layers.push(mapTypes[layer]);
        }
        if (bonusLayers != undefined && bonusLayers != null) {
            for (var i = 0; i < bonusLayers.length; i++) {
                layers.push(bonusLayers[i].name);
            }
        }
        return layers;
    };

    /**
     * Map control style
     * @param vars
     */
    this.getMapLayersControlStyle = function (vars) {
        if (vars.mtc === 'standard') {
            return google.maps.MapTypeControlStyle.DEFAULT;
        } else if (vars.mtc === 'drop_down') {
            return google.maps.MapTypeControlStyle.DROPDOWN_MENU;
        } else if (vars.mtc === 'bar') {
            return google.maps.MapTypeControlStyle.HORIZONTAL_BAR;
        }
    };

    /**
     * Map zoom control style
     * @param vars
     */
    this.getZoomControlStyle = function (vars) {
        if (vars.zoom === 'default') {
            return google.maps.ZoomControlStyle.DEFAULT;
        } else if (vars.zoom === 'large') {
            return google.maps.ZoomControlStyle.LARGE;
        } else if (vars.zoom === 'small') {
            return google.maps.ZoomControlStyle.SMALL;
        }
    };

    /**
     * Default map layer
     * @param mapid
     */
    this.defaultMapTypeId = function(mapid) {
        var mapTypes = {
            'map' : google.maps.MapTypeId.ROADMAP,
            'satellite' : google.maps.MapTypeId.SATELLITE,
            'hybrid' : google.maps.MapTypeId.HYBRID,
            'physical' : google.maps.MapTypeId.TERRAIN
        };
        var defaultLayer = mapTypes[Drupal.settings.tagmap[mapid].defaultlayer];
        return defaultLayer == null ? Drupal.settings.tagmap[mapid].defaultlayer : defaultLayer;
    };

    /**
     * Create text OverlayView
     * @param position
     * @param opts
     * @param map
     */
    this.createMarker = function(position, opts, map) {
        return new TextMarkerOverlay(position, opts, map);
    };

    this.addBonusLayers = function(mapid, map, defaultId) {
        var default_set = false;
        var layers = Drupal.settings.tagmap[mapid].layer;
        var defaultlayer = Drupal.settings.tagmap[mapid].defaultlayer;
        if (layers != undefined && layers != null) {
            for (var i = 0; i < layers.length; i++) {
                map.mapTypes.set(layers[i].name, mbLayer({tileset: layers[i].name, name: layers[i].title}));
                if (defaultlayer == layers[i].name && !default_set) {
                    map.setMapTypeId(layers[i].name);
                    default_set = true;
                }
            }
        }
        var overlays = Drupal.settings.tagmap[mapid].overlay;
        if (overlays != undefined && overlays != null) {
            for (var i = 0; i < overlays.length; i++) {
                var customoverlay = mbLayer({tileset: overlays[i].name, name: overlays[i].title});
                map.overlayMapTypes.push(customoverlay);
                var button = document.createElement("button");
                button.innerHTML = overlays[i].title;
                button.style.position = "absolute";
                button.style.top = "4px";
                button.style.right = "330px";
                button.className = "overlay-button hilight"
                button.style.zIndex = 70;
                map.getDiv().appendChild(button);
                button.onclick = function() {
                    if (button.className == "overlay-button hilight") {
                        map.overlayMapTypes.setAt(0, null);
                        button.className = "overlay-button lolight";
                    } else {
                        map.overlayMapTypes.setAt(0, customoverlay);
                        button.className = "overlay-button hilight";
                    }
                }
            }
        }
    };

    this.initialize = function() {
        var obj = this;
        var mapid = obj.id.split('-');
        mapid.pop();
        mapid.shift();
        var tmp = new String(mapid);
        var ind = tmp.indexOf("map");
        var mapnum_id = tmp.substr(4, ind - 4);
        mapid = mapid.join('-');
        obj.vars = Drupal.settings.tagmap[mapid];
        obj.vars.initial_state.zoom = parseInt(obj.vars.initial_state.zoom);
        var map = new google.maps.Map(document.getElementById(obj.id), Drupal.tagmap.setMapOptions(obj.vars, mapid));
        if (obj.vars.initial_state.zoom == -1) {
            //map.setZoom(map.fitBounds(bounds));
            map.setZoom(10);
        } else {
            map.setZoom(obj.vars.initial_state.zoom);
        }
        Drupal.tagmap.addBonusLayers(mapid, map, map.getMapTypeId());

        // Generation of Text markers
        var cluster = [];
        gmarkers[mapnum_id] = new Array();

        var bounds = new google.maps.LatLngBounds();
        for (var i = 0; i < obj.vars.markers.length; i++) {
            var opts = {};
            opts.text = obj.vars.markers[i].term;
            opts.weight = obj.vars.markers[i].weight;
            opts.infoText = obj.vars.markers[i].infoText;
            opts.infoTitle = obj.vars.markers[i].infoTitle;
            opts.infoMultimedia = obj.vars.markers[i].infoMultimedia;
            opts.minFontSize = obj.vars.markers[i].minFontSize;
            opts.fontStyle = obj.vars.markers[i].fontStyle;
            opts.markerColor = obj.vars.markers[i].markerColor;
            var marker = Drupal.tagmap.createMarker(
                    new google.maps.LatLng(obj.vars.markers[i].latitude, obj.vars.markers[i].longitude),
                    opts,
                    map);
            if (obj.vars.clustering == 'markerclusterer_compiled') {
                cluster.push(marker);
            }

            bounds.extend(new google.maps.LatLng(obj.vars.markers[i].latitude, obj.vars.markers[i].longitude));
        }
        if (obj.vars.clustering == 'markerclusterer_compiled') {
            var size = parseInt(obj.vars.cluster_size, 10);
            var style = parseInt(obj.vars.cluster_style, 10);
            size = size == -1 ? null : size;
            style = style == -1 ? null : style;
            var markerClusterer = new MarkerClusterer(map, cluster, {
                gridSize: size,
                styles: Drupal.tagmap.getClustererStyle(style, obj.vars.cluster_style_path)
            });
        }

        // If the default map center coordinates aren't provided set
        // the map center based on the markers position on the map.
        if (!obj.vars.initial_state.center) {
            map.fitBounds(bounds);
        }
    };

    this.embed = function(id, rel, overwrite) {
        var mapid = rel.split('-');
        mapid.pop();
        mapid.shift();
        var width = Drupal.settings.tagmap[mapid].width;
        var height = Drupal.settings.tagmap[mapid].height;
        var scripts = document.getElementsByTagName('script');
        var scrString = "";
        for (var i = 0; i < scripts.length; i++) {
            var tmp = scripts.item(i).text;
            if (tmp.search('tagmap') != -1 && tmp.search('infoText') != -1)
                scrString = tmp;

        }
        var path = Drupal.settings.basePath + 'tagmap/embed';
        jQuery.ajax({
            type : "POST",
            url : path,
            data : "id=" + id + "&rel=" + rel + "&clustering="
                    + Drupal.settings.tagmap[mapid].clustering
                    + "&settings=" + scrString + "&width=" + width
                    + "&height=" + height + "&storage=" + Drupal.settings.tagmap[mapid].storage + "&title=" + Drupal.settings.tagmap[mapid].tagmap_title
                    + "&vidiLink=" + Drupal.settings.tagmap[mapid].vidi_link + "&viewLink=" + Drupal.settings.tagmap[mapid].view_link + "&overwrite=" + overwrite,
            success : function(msg) {
                var divid = jQuery('#' + id).parent().attr('id');
                jQuery('#' + divid).hide('fast');
                jQuery('#' + divid).html(msg);
                jQuery('#' + divid).fadeIn('slow');
            },
            error : function(msg) {
                alert('greska');
            }
        });
    };
};

Drupal.behaviors.tagmap = {
    attach: function(context) {
        jQuery('.tagmap-control:not(.tagmap-processed)', context).addClass(
                'tagmap-processed').each(Drupal.tagmap.initialize);
        jQuery('.tagmap-embed-link', context).click(function() {
            Drupal.tagmap.embed(jQuery(this).attr('id'), jQuery(this).attr('rel'), jQuery(this).attr('overwrite'))
        });
    }
}