TextMarkerOverlay.prototype = new google.maps.OverlayView();

TextMarkerOverlay.prototype.infoWindow = new google.maps.InfoWindow();

function TextMarkerOverlay(coordinates, options, map) {
    this.pos = coordinates;
    options = options || {};
    this.tag_ = options.text || "";
    if (options.text) {
        options.text = undefined;
    }

    this.infoText = options.infoText;
    this.infoTitle = options.infoTitle;
    this.infoMultimedia = options.infoMultimedia;

    this.options = options;

    this.minFontSize = options.minFontSize || "12";
    this.fontSizeRatio = options.fontSizeRatio || 0.25;

    this.wrapperID = options.id || "textmarker";
    this.weight = options.weight || 1;
    this.fontStyle = options.fontStyle;
    this.markerColor = options.markerColor;

    this.wrapper = null;

    this.map_ = map;

    this.setMap(map);
}

TextMarkerOverlay.prototype.prepareWrapper = function() {

    this.wrapper = document.createElement("div");
    this.wrapper.style.fontSize = (((this.weight - 1) * this.fontSizeRatio + 1) * this.minFontSize) + "px";
    this.wrapper.style.fontStyle = this.fontStyle;
    this.wrapper.style.position = "absolute";
    this.wrapper.style.display = "inline";
    this.wrapper.style.cursor = "pointer";
    this.wrapper.style.whiteSpace  = "nowrap";

    if (this.markerColor != 'css')
        this.wrapper.style.color = this.markerColor;
    this.wrapper.innerHTML = this.tag_;
    if (this.isIE() === true && this.getInternetExplorerVersion() < 8) {
        this.wrapper.setAttribute('className', this.wrapperID + ' level' + this.weight + ' ' + this.wrapperID + '_body'); // ADD CLASSNAME BASED ON WEIGHT
    }
    else {
        this.wrapper.setAttribute('class', this.wrapperID + ' level' + this.weight + ' ' + this.wrapperID + '_body'); // ADD CLASSNAME BASED ON WEIGHT
    }
};

TextMarkerOverlay.prototype.onAdd = function() {
    this.prepareWrapper();
    var panes = this.getPanes();
    panes.floatPane.appendChild(this.wrapper);
};

TextMarkerOverlay.prototype.getPosition = function() {
    return this.pos;
}

TextMarkerOverlay.prototype.draw = function() {
    var me = this;
    var overlayProjection = this.getProjection();
    var position = overlayProjection.fromLatLngToDivPixel(this.pos);
    this.wrapper.style.left = (position.x - (this.wrapper.offsetWidth/2)) + 'px';
    this.wrapper.style.top = (position.y - (this.wrapper.offsetHeight/2)) + 'px';

    google.maps.event.addDomListener(this.wrapper, "click", function () {
        me.infoWindow.setOptions({maxWidth: me.map_.getDiv().style.width / 2});
        me.infoWindow.setPosition(me.pos);
        me.infoWindow.setContent(me.infoTitle + me.infoMultimedia + me.infoText);
        me.infoWindow.open(me.map_);
    });

    var panes = this.getPanes();
    panes.floatPane.appendChild(this.wrapper);
};

TextMarkerOverlay.prototype.onRemove = function() {
    this.wrapper.parentNode.removeChild(this.wrapper);
    this.wrapper = null;
};

// detects IE
TextMarkerOverlay.prototype.isIE = function () {
    return (navigator.userAgent.toLowerCase().indexOf('msie') !== -1);
};