<?php
/**
 * Form for admministration of embeded files
 * and confirmation form
 * @param $form_state
 */
function tagmap_admin_embeds_controller($form, &$form_state) {
  $checked = FALSE;
  if (!empty($form_state['values']['visualizations'])) {
    foreach ($form_state['values']['visualizations'] as $value) {
      if ($value) {
        $checked = TRUE;
        break;
      }
    }
  }
  if (!empty($form_state['values']['op']) && $form_state['values']['op'] == 'Delete selected' && $checked)
    return _tagmap_admin_embeds_delete_confirm($form, $form_state);

  return _tagmap_admin_embeds_form($form, $form_state);
}

/**
 * Confirmation form for deleting the embed file
 * and corresponding database records
 * @param $form_state
 */
function _tagmap_admin_embeds_delete_confirm($form, &$form_state) {
  $desc = 'This action CAN NOT be undone.';
  // Tell the submit handler to process the form
  $form['process'] = array('#type' => 'hidden', '#value' => 'true');
  // Make sure the form redirects in the end
  $form['destination'] = array('#type' => 'hidden', '#value' => 'admin/settings/tagmap/embeds');
  $to_delete = array();
  foreach ($form_state['values']['visualizations'] as $key => $value) {
    if ($value) {
      $to_delete[] = $value;
    }
  }
  $form['embeds_to_delete'] = array('#type' => 'hidden', '#value' => implode(',', $to_delete));
  return confirm_form($form,
                      t('Are you sure you want to delete the selected embed files?'),
                      'admin/settings/tagmap/embeds',
                      $desc,
                      'Delete',
                      'Return');
}

/**
 * Theme function for the the embed files table
 * @param $form
 */
function theme_tagmap_get_my_visualizations($form) {
  $form = $form['form'];
  $select_header = theme('table_select_header_cell');
  $header = array(
    $select_header,
    t('Title'),
    t('Embed code'),
    t('Date published'),
    t('Operations'),
  );
  $output = '';
  $rows = array();
  $counter = 0;
  if (isset($form['title'])) {
    foreach ($form['title'] as $key => $value) {
      if (is_int($key)) {
        $row = array();
        $row[] = drupal_render($form['visualizations'][$key]);
        $row[] = drupal_render($form['title'][$key]);
        $row[] = drupal_render($form['code'][$key]);
        $row[] = drupal_render($form['date'][$key]);
        $row[] = drupal_render($form['operations'][$key]);
        $rows[] = $row;
        $counter++;
      }
    }
  }
  if ($counter == 0) {
    $rows[] = array(array('data' => t('No posts available.'), 'colspan' => '5'));
  }

  $output .= theme('table', array('header' => $header, 'rows' => $rows));
  if ($form['pager']['#description']) {
    $output .= drupal_render($form['pager']);
  }
  return $output;
}

/**
 * Form for the administration of user created embed files.
 * Everyone who can make an embed file can administer their files.
 * @param $form_state
 */
function _tagmap_admin_embeds_form($form, &$form_state) {
  $limit = 20;

  $script = "function popup(mylink, windowname){if (! window.focus)return true; var href; if (typeof(mylink) == 'string') href=mylink; else href=mylink.href; window.open(href, windowname, 'width=1000,height=800,scrollbars=yes');return false;}";
  drupal_add_js($script, 'inline');

  $query = db_select('tagmap_embeds', 'te')
          ->fields('te', array('fid', 'title', 'link_vidi', 'link_view', 'visualisation_code'))
          ->fields('f', array('filename', 'uri', 'timestamp'))
          ->extend('PagerDefault')
          ->limit($limit);
  $query->leftJoin('file_managed', 'f', 'te.fid = f.fid');

  if (user_access('administer embeds')) {
    $form['filter'] = array(
      '#type' => 'fieldset',
      '#title' => t('Fiter embeds'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    );
    $results = db_query('SELECT uid, name FROM {users}');
    $users = array();
    foreach ($results as $result) {
      $users[$result->uid] = $result->uid != 0 ? $result->name : "anonymous";
    }

    $form['filter']['per_user'] = array(
      '#type' => 'select',
      '#multiple' => TRUE,
      '#default_value' => isset($_SESSION['tagmap_embed_filters']) ? $_SESSION['tagmap_embed_filters'] : '',
      '#title' => t('Fiter embeds'),
      '#options' => $users
    );

    $form['filter_button'] = array(
      '#type' => 'submit',
      '#value' => t('Filter'),
    );

    $form['filter_button_clear'] = array(
      '#type' => 'submit',
      '#value' => t('Clear filters'),
    );

    if (!empty($_SESSION['tagmap_embed_filters'])) {
      $conditions = db_or();
      foreach ($_SESSION['tagmap_embed_filters'] as $value) {
        $conditions->condition('f.uid', $value);
      }
      $query->condition($conditions);
    }
  }
  else {
    global $user;
    $query->condition('f.uid', $user->uid);
  }
  $results = $query->execute();
  $visualizations = array();
  foreach ($results as $result) {
    $visualizations[$result->fid] = '';
    $form['data']['title'][$result->fid] = array('#type' => 'item', '#description' => l($result->title, file_create_url($result->uri), array('attributes' => array('onClick' => 'return popup(this, \'My visualization\')'))));
    $form['data']['date'][$result->fid] = array('#type' => 'item', '#description' => format_date($result->timestamp));
    $form['data']['code'][$result->fid] = array('#type' => 'item', '#description' => $result->visualisation_code, '#prefix' => '<div style="font-size:11px;">', '#suffix' => '</div>');
    $vidilink = FALSE;
    $op_str = '';
    if (module_exists('vidi') && $result->link_vidi) {
      $op_str = l('edit embed', $result->link_vidi, array('query' => array('fid' => $result->fid)));
      $vidilink = TRUE;
    }
    if ($result->link_view) {
      $exists = FALSE;
      $link_arr = explode('/', $result->link_view);
      if ($view = views_get_view($link_arr[4])) {
        if (array_key_exists($link_arr[5], $view->display))
          $exists = TRUE;
      }

      if ($vidilink && $exists) {
        $op_str .= ' | ';
      }
      if ($exists && user_access('administer views'))
        $op_str .= l('edit view', $result->link_view);
    }
    $form['data']['operations'][$result->fid] = array('#type' => 'item', '#description' => $op_str);
  }

  $form['data']['visualizations'] = array('#type' => 'checkboxes', '#options' => $visualizations);
  $form['data']['pager'] = array('#type' => 'item', '#description' => theme('pager'));
  $form['data']['#theme'] = 'tagmap_get_my_visualizations';
  $form['data']['#prefix'] = '<div id="tagmap-embed-table-ajax">';
  $form['data']['#suffix'] = '</div>';
  $form['tagmap_delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete selected'),
  );
  return $form;
}

/**
 * Submit handler for the embed administration form.
 * @param $form
 * @param $form_state
 */
function tagmap_admin_embeds_controller_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#id'] != 'edit-filter-button' && $form_state['clicked_button']['#id'] != 'edit-filter-button-clear') {
    if (isset($form_state['values']['embeds_to_delete'])) {
      $for_deleting = explode(',', $form_state['values']['embeds_to_delete']);
      $txn = db_transaction();
      foreach ($for_deleting as $value) {
        $file = file_load($value);
        if (!file_delete($file)) {
          drupal_set_message(t('An error occurred while deleting the file fid @id from the file_managed table!', array('@id' => $value)), 'error');
          $txn->rollback();
          return TRUE;
        }
        $sql = "DELETE FROM {tagmap_embeds} WHERE fid = :fid";
        if (!db_query($sql, array(':fid' => $value))) {
          drupal_set_message(t('An error occurred while deleting the record with fid @id from the tagmap_embeds table!', array('@id' => $value)), 'error');
          $txn->rollback();
          return TRUE;
        }
        drupal_set_message(t('The file @name was deleted successfully!', array('@name' => $file->filename)), 'status');
      }
      return TRUE;
    }
    $form_state['rebuild'] = TRUE;
  }
  elseif ($form_state['clicked_button']['#id'] == 'edit-filter-button') {
    $_SESSION['tagmap_embed_filters'] = $form_state['values']['per_user'];
  }
  elseif ($form_state['clicked_button']['#id'] == 'edit-filter-button-clear') {
    unset($_SESSION['tagmap_embed_filters']);
  }
}

/**
 * Adds the google map js in the embed file.
 */
function _tagmap_add_google_map_embed() {
  $query = array(
    'sensor' => 'false'
  );
  return '<script src="' . check_url(url('http://maps.google.com/maps/api/js', array('query' => $query))) . '" type="text/javascript"></script>';
}

/**
 * @param $uid
 * @param $filename. The name of the embed file.
 * @param $filepath. Relative path of the embed file.
 * @param $storage. The state of the form from the VIDI module. Used to populate the form with correct data.
 * @param $vidi_link. link to the VIDI module form.
 * @param $view_link. Link to the corresponding view and display.
 * @param $result. Embed code.
 * @param $update. Is it an update or a creation of a new embed file.
 * @param $title. The title of the visualization. Displayed on the embed administration form.
 */
function _tagmap_save_file_data($uid, $file, $storage, $vidi_link, $view_link, $result, $update, $title) {
  $tagmap_embed_data = new stdClass();
  $tagmap_embed_data->fid = $file->fid;
  $tagmap_embed_data->title = $title;
  $tagmap_embed_data->visualisation_code = $result;

  if (!empty($storage)) {
    $tagmap_embed_data->form_state_storage = $storage;
  }
  if (!empty($vidi_link)) {
    $tagmap_embed_data->link_vidi = $vidi_link;
  }
  if (!empty($view_link)) {
    $tagmap_embed_data->link_view = $view_link;
  }
  if (!$update) {
    drupal_write_record('tagmap_embeds', $tagmap_embed_data);
  }
  else {
    drupal_write_record('tagmap_embeds', $tagmap_embed_data, 'fid');
  }

}

/**
 * Ajax calback that makes the embed file,
 * inserts corresponding data in the database
 * and returns the embed code to the user.
 */
function tagmap_embed() {
  global $user;
  global $base_url;
  $rel = $_POST['rel'];
  $title = $_POST['title'];
  $clustering = $_POST['clustering'];
  $settings = $_POST['settings'];
  if (empty($rel))
    return;
  $css = '<link type="text/css" rel="stylesheet" media="all" href="' . $base_url . '/' . drupal_get_path('module', 'tagmap') . '/tagmap.css"/>';
  $js = '<script type="text/javascript" src="' . $base_url . '/misc/jquery.js"></script>';
  $js .= '<script type="text/javascript" src="' . $base_url . '/misc/drupal.js"></script>';
  $js .= _tagmap_add_google_map_embed();
  $js .= '<script src="' . check_url(url('http://js.mapbox.com/g/' . TAGMAP_GOOGLE_API_VERSION . '/mapbox.js')) . '" type="text/javascript"></script>';
  $js .= '<script type="text/javascript" src="' . $base_url . '/' . drupal_get_path('module', 'tagmap') . '/js/tagmap_markers.js"></script>';
  $js .= '<script type="text/javascript" src="' . $base_url . '/' . drupal_get_path('module', 'tagmap') . '/js/tagmap.js"></script>';

  if ($clustering == 'markerclusterer_compiled')
    $js .= '<script type="text/javascript" src="' . $base_url . '/' . drupal_get_path('module', 'tagmap') . '/js/markerclusterer_compiled.js"></script>';
  $js .= '<script type="text/javascript">' . $settings . '</script>';
  $output = '<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" />' . $css . $js . '<style>img, a img {border:none}</style></head>';
  $output .= '<body><div class="dropdown-processed"><div id="' . $rel . '" class="tagmap-embed tagmap-control" style="width:' . $_POST['width'] . '; height:' . $_POST['height'] . '">' . t('Javascript is required to view this map.') . '</div></div></body>';
  $output .= '</html>';
  $directory = file_build_uri('tagmap_embed');
  file_prepare_directory($dir, FILE_CREATE_DIRECTORY);
  $directory = file_build_uri('tagmap_embed/' . $user->uid);
  file_prepare_directory($directory, TRUE);
  if (empty($_POST['overwrite'])) {
    $filename = 'tagmap_embed' . '.html';
    $file = file_save_data($output, $directory . '/' . $filename, FILE_EXISTS_RENAME);
    $update = FALSE;
  }
  else {
    $filename = $_POST['overwrite'] . '.html';
    $file = file_save_data($output, $directory . '/' . $filename, FILE_EXISTS_REPLACE);
    $update = TRUE;
  }
  $result = "<div class=\"embed-instructions\"><strong>Embed link:</strong><br/>";
  $result .= "<div class=\"embed-code\" style=\" color: #666;\">";
  $code = "&lt;iframe scrolling = \"no\" width=\"" . $_POST['width'] . "\" height=\"" . $_POST['height'] . "\" src=\"";
  $code .= file_create_url($file->uri);
  $code .= "\"&gt;&lt;/iframe&gt";
  $result .= $code . "</div></div>";
  _tagmap_save_file_data($user->uid, $file, $_POST['storage'], $_POST['vidiLink'], $_POST['viewLink'], $code, $update, $title);
  echo $result;
}
