<?php
class views_plugin_style_tagmap extends views_plugin_style {

  /**
   * Set default options
   */
  function option_definition() {
    $options = parent::option_definition();
    $options['bands']['contains'] = array(
      'focus' => array('default' => 'today'),
      'band1_unit' => array('default' => 'day'),
      'band2_unit' => array('default' => 'month'),
    );
    $options['datasource'] = array(
      'default' => 'basic',
    );
    $options['pathcolor'] = array(
      'default' => 'red',
    );
    $options['display']['contains'] = array(
      'width' => array('default' => variable_get('timelinemap_default_width', '100%')),
      'tlheight' => array('default' => variable_get('timelinemap_default_tlheight', '400px')),
      'mheight' => array('default' => variable_get('timelinemap_default_mheight', '400px')),
    );
    $options['embed_link'] = array(
      'default' => '',
    );


    $options['steps'] = array('default' => '6');
    return $options;
  }

  function render($values) {
    if (empty($this->row_plugin)) {
      vpr('Missing row plugin');
      return;
    }
    $lat_field = $this->view->display_handler->get_handler('field', $this->options['latfield'])->real_field;
    $lon_field = $this->view->display_handler->get_handler('field', $this->options['lonfield'])->real_field;
    $info_title_field = $this->view->display_handler->get_handler('field', $this->options['description_title'])->real_field;
    $info_html_field = $this->view->display_handler->get_handler('field', $this->options['description'])->real_field;
    $info_multimedia_field = $this->view->display_handler->get_handler('field', $this->options['imagefield'])->real_field;
    $tid_field = 'tid';
    $term_name_field = ($this->view->display_handler->get_handler('field', $this->options['tagfield'])->real_field ?
            $this->view->display_handler->get_handler('field', $this->options['tagfield'])->real_field : 'term_data_name');
    if ($this->options['weightfield']) {
      $weight_field = $this->view->display_handler->get_handler('field', $this->options['weightfield'])->real_field;
    }
    $results = array();
    foreach ($this->view->result as $rkey => $result) {
      foreach ($this->view->field as $fkey => $field) {
        $results[$rkey][$fkey] = $this->view->render_field($fkey, $rkey);
      }
    }
    $output = '';
    foreach ($results as $result) {
      if (!$weight_field) {
        $weight = tagmap_term_count_nodes($result[$tid_field]);
      }
      else {
        $weight = $result[$weight_field];
      }

      if ($weight || $weight == 0) {
        $markers[] = array(
          'latitude' => $result[$lat_field],
          'longitude' => $result[$lon_field],
          'term' => $result[$term_name_field],
          'weight' => $weight,
          'infoTitle' => '<div class="tagmap-info-title">' . $result[$info_title_field] . '</div>',
          'infoMultimedia' => '<div class="tagmap-info-multimedia">' . $result[$info_multimedia_field] . '</div>',
          'infoText' => '<div class="tagmap-info-content">' . $result[$info_html_field] . '</div>',
          'minFontSize' => $this->options['fontsize'],
          'fontStyle' => $this->options['fontstyle'],
          'markerColor' => 'css',
        );
      }
      else if (empty($weight)) {
        drupal_set_message(t('The tagmap could not be shown because a weight field is not set.'), 'error');
      }
    }

    tagmap_generate_weighted_markers($markers, $this->view->style_plugin->options['steps'], $this->options['color']);

    if (!empty($markers)) {
      $map['markers'] = $markers;
      $map['embed_link'] = $this->view->style_plugin->options['map_settings']['embed_link'];
      if (isset($this->view->style_plugin->options['map_settings']['width'])) {
        $map['width'] = $this->view->style_plugin->options['map_settings']['width'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['height'])) {
        $map['height'] = $this->view->style_plugin->options['map_settings']['height'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['clustering']) && $this->view->style_plugin->options['map_settings']['clustering'] == 'markerclusterer_compiled') {
        $map['clustering'] = $this->view->style_plugin->options['map_settings']['clustering'];
        $map['cluster_style'] = $this->view->style_plugin->options['map_settings']['cluster_style'];
        $map['cluster_size'] = $this->view->style_plugin->options['map_settings']['cluster_size'];
        $map['cluster_style_path'] = $this->view->style_plugin->options['map_settings']['cluster_style_path'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['initial_state']['zoom'])) {
        $map['initial_state']['zoom'] = $this->view->style_plugin->options['map_settings']['initial_state']['zoom'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['initial_state']['center'])) {
        $map['initial_state']['center'] = $this->view->style_plugin->options['map_settings']['initial_state']['center'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['layers']['layer'])) {
        $map['layers']['layer'] = $this->view->style_plugin->options['map_settings']['layers']['layer'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['layers']['overlay'])) {
        $map['layers']['overlay'] = $this->view->style_plugin->options['map_settings']['layers']['overlay'];
      }
      if (isset($this->view->style_plugin->options['map_settings']['layers']['defaultlayer'])) {
        $map['layers']['defaultlayer'] = $this->view->style_plugin->options['map_settings']['layers']['defaultlayer'];
      }
      $map['view_link'] = 'admin/structure/views/view/' . $this->view->name . '/' . $this->view->current_display;
      $map['tagmap_title'] = $this->view->style_plugin->options['title'];
      if (module_exists('vidi')) {
        $count = db_query("SELECT COUNT(*) FROM {vidi_visualization_tables} WHERE tablename=:tablename",
                          array('tablename' => $this->view->base_table))->fetchField();
        if ($count) {
          $storage = array_merge($this->view->style_plugin->options, array('tablename' => $this->view->base_table));
          foreach ($storage['map_settings'] as $key => $value) {
            $storage[$key] = $value;
          }
          foreach ($storage['map_settings']['initial_state'] as $key => $value) {
            $storage[$key] = $value;
          }
          foreach ($storage['map_settings']['layers'] as $key => $value) {
            $storage[$key] = $value;
          }
          unset($storage['map_settings']);
          unset($storage['initial_state']);
          unset($storage['layers']);
          $map['storage'] = serialize($storage);
          $map['vidi_link'] = 'vidi/tagmap_form/' . $this->view->base_table;
        }
        else {
          $map['storage'] = '';
          $map['vidi_link'] = '';
        }
      }
      else {
        $map['storage'] = '';
        $map['vidi_link'] = '';
      }
      $output .= theme($this->theme_functions(), array('view' => $this->view, 'options' => $this->options, 'rows' => $map));
    }
    else {
      drupal_set_message(t('The tagmap has no markers to display. Choose some content to see the tagmap.'), 'status');
    }
    return $output;
  }

  /**
   * Render the given style.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    module_load_include('inc', 'tagmap', 'tagmap_color_presets');
    $tagmap_color_presets = _tagmap_color_presets();
    $preset_start = array('' => 'From CSS file');
    $defaults = array_merge(tagmap_defaults(), variable_get('tagmap_settings', array()));
    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Name your tagmap'),
      '#reqiured' => TRUE,
      '#default_value' => isset($this->options['title']) ? $this->options['title'] : '',
      '#maxlength' => 60,
      '#size' => 60,
    );

    $form['map_settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('Map settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['map_settings']['width'] = array(
      '#type' => 'textfield',
      '#title' => t('Width'),
      '#description' => t('Map width, as a CSS length or percentage. Examples: <em>50px</em>, <em>5em</em>, <em>2.5in</em>, <em>95%</em>.'),
      '#default_value' => (isset($this->options['map_settings']['width']) ? $this->options['map_settings']['width'] : $defaults['width']),
      '#maxlength' => 10,
      '#size' => 10,
      '#required' => TRUE,
    );

    $form['map_settings']['height'] = array(
      '#type' => 'textfield',
      '#title' => t('Height'),
      '#description' => t('Map height, as a CSS length or percentage. Examples: <em>50px</em>, <em>5em</em>, <em>2.5in</em>, <em>95%</em>.'),
      '#default_value' => (isset($this->options['map_settings']['height']) ? $this->options['map_settings']['height'] : $defaults['height']),
      '#maxlength' => 10,
      '#size' => 10,
      '#required' => TRUE,
    );

    $form['map_settings']['initial_state'] = array(
      '#type' => 'fieldset',
      '#title' => t('Set the initial state of the map'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => t('These settings are optional. If you do not enter the values, they will be calculated on the basis of the markers position on the map.'),
    );

    $form['map_settings']['initial_state']['zoom'] = array(
      '#type' => 'select',
      '#title' => t('Default zoom'),
      '#default_value' => (isset($this->options['map_settings']['initial_state']['zoom']) ? $this->options['map_settings']['initial_state']['zoom'] : $defaults['initial_state']['zoom']),
      '#options' => array(-1 => t('None')) + drupal_map_assoc(range(0, 17)),
      '#description' => t('The default zoom level of the map.'),
    );

    $form['map_settings']['initial_state']['center'] = array(
      '#type' => 'textfield',
      '#title' => t('Default center'),
      '#default_value' => (isset($this->options['map_settings']['initial_state']['center']) ? $this->options['map_settings']['initial_state']['center'] : $defaults['initial_state']['center']),
      '#size' => 50,
      '#maxlength' => 255,
      '#description' => t('The default center coordinates of the map, expressed as a decimal latitude and longitude, separated by a comma.'),
      '#element_validate' => array('tagmap_center_validate'),
    );

    $form['map_settings']['layers'] = array(
      '#type' => 'fieldset',
      '#title' => t('Add additional layers and overlays to the map'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#description' => t('Here you can add additional layers and/or overlays to the map. You have to define them ahead of time on the !administration page',
                          array('!administration' => user_access('administer site configuration') ?
                                  l('administration', 'admin/settings/tagmap/layers/addlayer', array('attributes' => array('target' => '_blank'))) :
                                  'administration')),
    );
    $results = db_query("SELECT * FROM {tagmap_layers}");
    $layers = array();
    $overlays = array('' => 'None');
    $default_layers = array();
    foreach ($results as $result) {
      if ($result->baselayer) {
        $layers[$result->lid] = $result->title;
        $default_layers[$result->name] = $result->title;
      }
      else
        $overlays[$result->lid] = $result->title;
    }
    $form['map_settings']['layers']['layer'] = array(
      '#type' => 'select',
      '#title' => t('Select layers to add'),
      '#multiple' => TRUE,
      '#options' => $layers,
      '#default_value' => isset($this->options['map_settings']['layers']['layer']) ? $this->options['map_settings']['layers']['layer'] : ''
    );
    $form['map_settings']['layers']['overlay'] = array(
      '#type' => 'select',
      '#title' => t('Select an overlay to add'),
      '#options' => $overlays,
      '#default_value' => isset($this->options['map_settings']['layers']['overlay']) ? $this->options['map_settings']['layers']['overlay'] : ''
    );

    foreach ($defaults['baselayers'] as $key => $value) {
      if ($value)
        $default_layers[$key] = $key;
    }

    $form['map_settings']['layers']['defaultlayer'] = array(
      '#type' => 'select',
      '#title' => t('Select the default map layer'),
      '#description' => t('Any additional layer MUST be selected in the multiple select above for this field to have an effect.'),
      '#options' => $default_layers,
      '#default_value' => isset($this->options['map_settings']['layers']['defaultlayer']) ? $this->options['map_settings']['layers']['defaultlayer'] : $defaults['maptype']
    );
    if (file_exists(drupal_get_path('module', 'tagmap') . '/js/markerclusterer_compiled.js')) {
      $opts = array('' => 'None');
      if (file_exists(drupal_get_path('module', 'tagmap') . '/js/markerclusterer_compiled.js'))
        $opts['markerclusterer_compiled'] = 'markerclusterer_compiled.js';
      $form['map_settings']['clustering'] = array(
        '#type' => 'select',
        '#title' => t('Use clusters for marker management'),
        '#options' => $opts,
        '#default_value' => (isset($this->options['map_settings']['clustering']) ? $this->options['map_settings']['clustering'] : $defaults['clustering']),
      );
      $form['map_settings']['cluster_style_path'] = array(
        '#type' => 'textfield',
        '#title' => t('Path or webpath to icon directory (with trailing slash)'),
        '#process' => array('ctools_dependent_process'),
        '#dependency' => array('edit-style-options-map-settings-clustering' => array('markerclusterer_compiled')),
        '#default_value' => (isset($this->options['map_settings']['cluster_style_path']) ?
                $this->options['map_settings']['cluster_style_path'] :
                'http://google-maps-utility-library-v3.googlecode.com/svn/trunk/markerclusterer/images/'),
      );
      $form['map_settings']['cluster_style'] = array(
        '#type' => 'select',
        '#title' => t('Style of the cluster icon'),
        '#process' => array('ctools_dependent_process'),
        '#dependency' => array('edit-style-options-map-settings-clustering' => array('markerclusterer_compiled')),
        '#options' => array('-1' => t('Default'), '0' => t('People'), '1' => t('Conversation'), '2' => t('Heart')),
        '#default_value' => isset($this->options['map_settings']['cluster_style']) ? $this->options['map_settings']['cluster_style'] : '-1',
      );
      $form['map_settings']['cluster_size'] = array(
        '#type' => 'select',
        '#title' => t('Cluster size'),
        '#process' => array('ctools_dependent_process'),
        '#dependency' => array('edit-style-options-map-settings-clustering' => array('markerclusterer_compiled')),
        '#default_value' => isset($this->options['map_settings']['cluster_size']) ? $this->options['map_settings']['cluster_size'] : '-1',
        '#options' => array('-1' => t('Default'), '0' => 40, '1' => 50, '2' => 70, '3' => 80),
      );
    }

    $form['map_settings']['embed_link'] = array(
      '#type' => 'checkbox',
      '#weight' => 4,
      '#title' => t('Display embed link'),
      '#return_value' => 1,
      '#default_value' => isset($this->options['map_settings']['embed_link']) ? $this->options['map_settings']['embed_link'] : '0',
    );

    $form['steps'] = array(
      '#type' => 'textfield',
      '#title' => t('Tag levels'),
      '#description' => t('Number of different tag levels that will be generated. Must be an integer value.'),
      '#default_value' => (isset($this->options['steps']) ? $this->options['steps'] : 6),
      '#maxlength' => 10,
      '#size' => 10,
      '#required' => TRUE,
    );

    $form['fontsize'] = array(
      '#type' => 'textfield',
      '#title' => t('Font size in pixels'),
      '#description' => t('Minimum font size for the first level terms.'),
      '#default_value' => (isset($this->options['fontsize']) ? $this->options['fontsize'] : 10),
      '#maxlength' => 10,
      '#size' => 10,
      '#required' => TRUE,
    );

    $font_style = array('normal' => 'Normal', 'italic' => 'Italic', 'oblique' => 'Oblique');

    $form['fontstyle'] = array(
      '#type' => 'select',
      '#title' => t('Font style'),
      '#default_value' => (isset($this->options['fontstyle']) ? $this->options['fontstyle'] : $font_style['normal']),
      '#options' => $font_style,
      '#required' => TRUE,
    );

    $form['color'] = array(
      '#type' => 'select',
      '#title' => t('Color preset'),
      '#default_value' => isset($this->options['color']) ? $this->options['color'] : $preset_start[''],
      '#options' => $preset_start + $tagmap_color_presets,
    );

    $supported_string_handlers = array('views_handler_field', 'views_handler_field_taxonomy', 'views_handler_field_markup');
    $supported_numeric_handlers = array('views_handler_field_numeric', 'geotaxonomy_handler_field_term_node_count', 'location_views_handler_field_latitude', 'location_views_handler_field_longitude');
    $options = array('numerical' => array(), 'string' => array(), 'unsorted' => array());
    $handlers = $this->display->handler->get_handlers('field');
    foreach ($handlers as $field => $handler) {
      $handler_class = get_class($handler);
      if (in_array($handler_class, $supported_numeric_handlers)) {
        $options['numerical'][$field] = $handler->ui_name();
      }
      elseif (in_array($handler_class, $supported_string_handlers)) {
        $options['string'][$field] = $handler->ui_name();
      }
      else
        $options['unsorted'][$field] = $handler->ui_name();
    }

    $form['latfield'] = array(
      '#title' => t('Latitude field'),
      '#description' => t('Format must be degrees decimal.'),
      '#type' => 'select',
      '#options' => array_merge($options['numerical'], $options['unsorted']),
      '#default_value' => $this->options['latfield'],
      '#required' => TRUE,
    );

    $form['lonfield'] = array(
      '#title' => t('Longitude field'),
      '#description' => t('Format must be degrees decimal.'),
      '#type' => 'select',
      '#options' => array_merge($options['numerical'], $options['unsorted']),
      '#default_value' => $this->options['lonfield'],
      '#required' => TRUE,
    );

    $form['weightfield'] = array(
      '#title' => t('Tag weight field'),
      '#description' => t('If you don\'t choose a field, term weight will be calculated counting number of nodes assigned with the term.'),
      '#type' => 'select',
      '#options' => ($this->view->base_table != 'term_data' ? array_merge($options['numerical'], $options['unsorted']) : array_merge(array('' => 'None'), $options['numerical'], $options['unsorted'])),
      '#default_value' => $this->options['weightfield'],
    );

    $form['description_title'] = array(
      '#title' => t('Info window title'),
      '#description' => t('The title of the description cloud.'),
      '#type' => 'select',
      '#options' => array_merge(array('' => 'None'), $options['string'], $options['unsorted']),
      '#default_value' => $this->options['description_title'],
    );

    $form['description'] = array(
      '#title' => t('Info window content'),
      '#description' => t('The content of the description cloud.'),
      '#type' => 'select',
      '#options' => array_merge(array('' => 'None'), $options['string'], $options['unsorted']),
      '#default_value' => $this->options['description'],
    );

    $form['imagefield'] = array(
      '#title' => t('Image field'),
      '#type' => 'select',
      '#options' => array_merge(array('--' => '--'), $options['string'], $options['unsorted']),
      '#default_value' => $this->options['imagefield'],
      '#description' => t('This field should be excluded from display so the path to the image doesn\'t render.'),
    );

    if ($this->view->base_table != 'term_data') {
      $form['tagfield'] = array(
        '#title' => t('Tag field'),
        '#description' => t('String that will be displayed over map.'),
        '#type' => 'select',
        '#options' => array_merge(array('' => ''), $options['string'], $options['unsorted']),
        '#default_value' => $this->options['tagfield'],
        '#required' => TRUE,
      );
    }
  }

  function options_validate(&$form, &$form_state) {
    parent::options_validate($form, $form_state);
    $maps = array('map', 'satellite', 'hybrid', 'physical');
    if (!in_array($form_state['values']['style_options']['map_settings']['layers']['defaultlayer'], $maps)) {

      $query = db_select('tagmap_layers')
              ->condition('name', $form_state['values']['style_options']['map_settings']['layers']['defaultlayer'], '=')
              ->fields('tagmap_layers');
      $results = $query->execute();
      $selected = FALSE;
      foreach ($results as $result) {
        if (in_array($result->lid, $form_state['values']['style_options']['map_settings']['layers']['layer']))
          $selected = TRUE;
      }
      if (!$selected)
        form_set_error('style_options][map_settings][layers][defaultlayer', t('If selecting an additional layer as default, it must be enabeled firs trough the multiple select field.'));
    }
    $title = trim($form_state['values']['style_options']['title']);
    if (empty($title)) {
      form_set_error('style_options][title', t('Field @field must not be empty.', array('@field' => $form['title']['#title'])));
    }

    if (!is_numeric($form_state['values']['style_options']['steps'])) {
      form_set_error('style_options][steps', t('Field @field must be integer.', array('@field' => $form['steps']['#title'])));
    }
    elseif ($form_state['values']['style_options']['steps'] <= 0) {
      form_set_error('style_options][steps', t('Field @field must be positive value.', array('@field' => $form['steps']['#title'])));
    }

    if (!is_numeric($form_state['values']['style_options']['fontsize'])) {
      form_set_error('style_options][fontsize', t('Field @field must be numeric.', array('@field' => $form['fontsize']['#title'])));
    }
    elseif ($form_state['values']['style_options']['fontsize'] <= 0) {
      form_set_error('style_options][fontsize', t('Field @field must be positive value.', array('@field' => $form['fontsize']['#title'])));
    }

    if (!tagmap_css_dimensions_validate($form_state['values']['style_options']['map_settings']['width'])) {
      form_set_error('style_options][map_settings][width', t('The specified value is not a valid CSS dimension.'));
    }

    if (!tagmap_css_dimensions_validate($form_state['values']['style_options']['map_settings']['height'], TRUE)) {
      form_set_error('style_options][map_settings][height', t('The specified value is not a valid CSS dimension or is a percentage.'));
    }
  }
}